package org.xiss.index.impl.persistence;

import java.util.HashMap;
import java.util.Map.Entry;

import net.sf.xanswer.storage.map.PersistentException;

/**
 * A simple fixed-capacity FIFO-cache.
 * On each access to an entry (get/put) it is put together with its value
 * into the cache, possibly causing the oldest-cached entry to be pushed out
 * of it. If an entry is not contained in the cache (on get or put) it is read from disk.
 * If a cache entry is not 'dirty' (it has not been changed since being in cache)
 * then nothing is done when it is pushed out, otherwise it is re-serialized and 
 * stored persistently. 
 * Entries are stored in a hash map and threaded into a linked list.
 */
class BTreeCache<K, V> {
	private HashMap<K, CacheEntry<K, V>> myCache;

	private GenericBTree<K, V> myTree;

	private int myCapacity;

	private int myCount;

	private CacheEntry<K, V> myHead;

	private CacheEntry<K, V> myTail;

	private int myHits, myMisses;

	private int myActualGets, myActualPuts;

	public BTreeCache(GenericBTree<K, V> tree, int capacity) {
		myTree = tree;
		myCapacity = capacity;
		myCache = new HashMap<K, CacheEntry<K, V>>(myCapacity);
	}

	public boolean containsInCache(K key) {
		return myCache.containsKey(key);
	}

	public void flush() throws PersistentException {
		for (Entry<K, CacheEntry<K, V>> entry : myCache.entrySet()) {
			if (entry.getValue().IsDirty) {
				++myActualPuts;
				myTree.putNoCache(entry.getKey(), entry.getValue().Value);
			}
		}
		myCount = 0;
		myHead = myTail = null;
		myCache.clear();
	}

	public V get(K key) throws PersistentException {
		CacheEntry<K, V> entry = myCache.get(key);
		if (entry != null) {
			++myHits;
			return entry.Value;
		} else {
			++myMisses;
			++myActualGets;
			V value = myTree.getNoCache(key);
			putIntoCache(key, value);
			return value;
		}
	}

	public int getCapacity() {
		return myCapacity;
	}

	public int getCount() {
		return myCount;
	}

	public int getHits() {
		return myHits;
	}

	public int getActualGets() {
		return myActualGets;
	}

	public int getActualPuts() {
		return myActualPuts;
	}

	public int getMisses() {
		return myMisses;
	}

	public void put(K key, V value) throws PersistentException {
		CacheEntry<K, V> entry = myCache.get(key);
		if (entry != null) {
			++myHits;
			entry.IsDirty = true;
			entry.Value = value;
		} else {
			++myMisses;
			putIntoCache(key, value);
		}
	}

	// Puts the value into cache regardless of whether it is already there
	private void putIntoCache(K key, V value) throws PersistentException {
		adjustToAllowedSize();
		CacheEntry<K, V> newTail = new CacheEntry<K, V>(key, value);
		newTail.IsDirty = true;
		if (myHead == null) {
			myHead = myTail = newTail;
		} else {
			myTail.Next = newTail;
			myTail = newTail;
		}
		myCache.put(key, newTail);
		++myCount;
	}

	private void adjustToAllowedSize() throws PersistentException {
		if (myCount == myCapacity) {
			removeHead();
		}
	}

	private void removeHead() throws PersistentException {
		if(myHead.IsDirty) {
			myTree.putNoCache(myHead.Key, myHead.Value);
			++myActualPuts;
		}
		myCache.remove(myHead.Key);
		myHead = myHead.Next;
		--myCount;
	}

	private static class CacheEntry<K, V> {
		public K Key;

		public V Value;

		public CacheEntry<K, V> Next;

		public boolean IsDirty;

		public CacheEntry(K key, V value) {
			Key = key;
			Value = value;
		}
	}
}
