package org.xiss.query;

import java.util.List;

/**
 * Result of a query represented as a sorted list of chunks. A chunk is a
 * segment of a path in a document defined by node ID's of its root and deep
 * part. For instance, in a path
 * /public/commerce/products/gadgets/screwdriver/@color the path from 'commerce'
 * to 'screwdriver' will be represented as a chunk having root pointing to
 * 'commerce' and 'deep' pointing to 'screwdriver'.
 * 
 * It is important that we represent the result not plainly as a list of node
 * ID's but as chunks because it enables us to use the same interface both for
 * final and intermediate query results. For example for a query like
 * "public/commerce/((products|sales)/gadgets)/screwdriver" there is an
 * intermediate query of "((products|sales)/gadgets)" whose result will contain
 * information not only about which such gadgets are there but also about to
 * products or sales node they belong, which is necessary for combining this
 * subquery with "public/commerce/".
 * 
 * The result can be sorted either by chunks root id, either by deep id.
 */
public interface IResult {
	/**
	 * Get list of chunks
	 */
	List<Chunk> getChunks();

	/**
	 * Returns whether this result's chunks are sorted by ID of their root
	 * element
	 */
	boolean isSortedByStart();

	/**
	 * Returns whether this result's chunks are sorted by ID of their deep
	 * element
	 */
	boolean isSortedByEnd();

	/**
	 * Sorts this result's chunks by their start. Supposes that the chunks are
	 * already almost sorted and uses simple bubblesort.
	 */
	void sortByStart();

	/**
	 * Sorts this result's chunks by their end. Supposes that the chunks are
	 * already almost sorted and uses simple bubblesort.
	 */
	void sortByEnd();

	/**
	 * Returns whether this result contains no chunks.
	 */
	boolean isEmpty();

	/**
	 * Creates a copy of this result
	 */
	IResult copy();
}
