package org.xiss.index.impl;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.LineNumberReader;

import junit.framework.Assert;
import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestResult;
import net.sf.xanswer.indexing.IndexException;
import net.sf.xanswer.storage.map.PersistentException;

import org.xiss.index.IXissIndex;
import org.xiss.index.XissIndexFactory;
import org.xiss.map.IBTree;
import org.xiss.map.KeyValuePair;

public class SingleFileIndexTest implements Test {
	private String myPath;

	private String myFileName;

	private String myIndexLocation;

	public SingleFileIndexTest(String path, String fileName,
			String indexLocation) {
		myPath = path;
		myFileName = fileName;
		myIndexLocation = indexLocation;
	}

	public int countTestCases() {
		return 1;
	}

	public void run(TestResult result) {
		result.startTest(this);
		try {
			IXissIndex index = null;
			String path = new File(myPath, myFileName + ".xml")
					.getAbsolutePath();
			try {
				index = XissIndexFactory.newInstance().open(myIndexLocation);
			} catch (IndexException e1) {
				e1.printStackTrace();
				result.addFailure(this, new AssertionFailedError(
						"Couldn't read index from disk: " + e1.toString()));
				return;
			}
			String actualIndexText = indexToString(index);
			String indexFileName = path.substring(0, path.length() - 4)
					+ ".txt";
			try {
				String expected = readFileToString(indexFileName);

				if (!expected.equals(actualIndexText)) {
					System.out.println("Expected: " + expected);
					System.out.println("Actual: " + actualIndexText);
					result.addFailure(this, new AssertionFailedError(
							"Index for " + myFileName
									+ " was not built correctly"));
				}
			} catch (IOException e) {
				result.addFailure(this, new AssertionFailedError("File "
						+ indexFileName + " couldn't be read: "
						+ e.getMessage()));
			}
		} catch (IndexException e) {
			e.printStackTrace();
			result.addFailure(this, new AssertionFailedError(e.getMessage()));
		} finally {
			result.endTest(this);
		}
	}

	private String readFileToString(String uri) throws IOException {
		File file = new File(uri);
		StringBuilder builder = new StringBuilder();

		LineNumberReader reader = new LineNumberReader(new FileReader(file));
		String line;
		while ((line = reader.readLine()) != null) {
			builder.append(line + "\n");
		}
		return builder.toString();
	}

	public void generateExpectedResult() {
		try {
			IXissIndex index = null;
			String path = new File(myPath, myFileName + ".xml")
					.getAbsolutePath();
			try {
				index = XissIndexFactory.newInstance().createNewFromDocument(
						path, myIndexLocation);
			} catch (IndexException e) {
				e.printStackTrace();
				Assert.fail("Couldn't read index from disk: " + e.toString());
			}
			String actualIndexText = indexToString(index);
			try {
				index.close();
			} catch (IndexException e) {
				// e.printStackTrace();
				throw new RuntimeException(e);
			}

			String indexFileName = path.substring(0, path.length() - 4)
					+ ".txt";
			FileWriter writer = null;
			try {
				writer = new FileWriter(indexFileName);
				writer.write(actualIndexText);
			} catch (IOException e) {
			} finally {
				if (writer != null) {
					try {
						writer.close();
					} catch (Exception ex) {
					}
				}

			}
		} catch (IndexException e) {
			throw new RuntimeException(e);
		}
	}

	private String indexToString(IXissIndex index) throws IndexException {
		try {
			StringBuilder sb = new StringBuilder();
			sb.append("Element index: \n");
			sb.append(btreeToString(index.getElementIndex())).append("\n");
			sb.append("Attribute index: \n");
			sb.append(btreeToString(index.getAttributeIndex())).append("\n");
			sb.append("Structure index: \n");
			sb.append(index.getStructureIndex().toString()).append("\n");
			sb.append("Name table: \n");
			sb.append(btreeToString(index.getNameTable())).append("\n");
			sb.append("Value table: \n");
			sb.append(btreeToString(index.getValueTable())).append("\n");
			return sb.toString();
		} catch (PersistentException e) {
			throw new IndexException(e);
		}
	}

	private <K, V> String btreeToString(IBTree<K, V> tree)
			throws PersistentException {
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		for (KeyValuePair<K, V> pair : tree) {
			sb.append(pair + ", ");
		}
		if (sb.length() != 1) {
			sb.delete(sb.length() - 2, sb.length());
		}
		sb.append("]");
		return sb.toString();
	}

	public String toString() {
		return myFileName;
	}
}
