package org.xiss.query;

import junit.framework.Assert;
import junit.framework.TestCase;

public class QueryParseTest extends TestCase {
	public void testParserSimple() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/");
			Assert.assertEquals(q.toString(), "ROOT");
			q = factory.parse("/elem");
			Assert.assertEquals(q.toString(), "ROOT SELECT(elem) JOIN(1)");
			q = factory.parse("/elem1/elem2");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem1) SELECT(elem2) JOIN(1) JOIN(1)");
			q = factory.parse("/elem1/elem2/elem3/elem4");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(elem1) SELECT(elem2) SELECT(elem3) SELECT(elem4) JOIN(1) JOIN(1) JOIN(1) JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}

	public void testSelectText() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/text()");
			Assert.assertEquals(q.toString(), "ROOT SELECT(text()) JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}

	public void testStrictJoin() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/elem1/_/elem2");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem1) SELECT(elem2) JOIN(2) JOIN(1)");
			q = factory.parse("/elem1/_/_/_/elem2");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem1) SELECT(elem2) JOIN(4) JOIN(1)");
			q = factory.parse("/elem1/_/_/_/elem2/elem3/_/elem4");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(elem1) SELECT(elem2) SELECT(elem3) SELECT(elem4) JOIN(2) JOIN(1) JOIN(4) JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}

	public void testDeepJoin() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/elem1/_*/elem2");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem1) SELECT(elem2) JOIN(1..INF) JOIN(1)");
			q = factory.parse("/elem1/_*/elem2/_*/elem3");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(elem1) SELECT(elem2) SELECT(elem3) JOIN(1..INF) JOIN(1..INF) JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}

	public void testUnionAndBraces() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/(elem1|elem2)");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem1) SELECT(elem2) UNION JOIN(1)");
			q = factory.parse("/elem1|/elem2");
			Assert
					.assertEquals(q.toString(),
							"ROOT SELECT(elem1) JOIN(1) ROOT SELECT(elem2) JOIN(1) UNION");
			q = factory.parse("/(elem1|elem2|elem3)");
			Assert
					.assertEquals(q.toString(),
							"ROOT SELECT(elem1) SELECT(elem2) UNION SELECT(elem3) UNION JOIN(1)");
			q = factory.parse("/(elem1)");
			Assert.assertEquals(q.toString(), "ROOT SELECT(elem1) JOIN(1)");
			q = factory.parse("(/elem1)");
			Assert.assertEquals(q.toString(), "ROOT SELECT(elem1) JOIN(1)");
			q = factory.parse("(((/elem1)))");
			Assert.assertEquals(q.toString(), "ROOT SELECT(elem1) JOIN(1)");
			q = factory.parse("(((/(((elem1))))))");
			Assert.assertEquals(q.toString(), "ROOT SELECT(elem1) JOIN(1)");
			q = factory.parse("(/abcd)|(/(elem1|elem2))");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(abcd) JOIN(1) ROOT SELECT(elem1) SELECT(elem2) UNION JOIN(1) UNION");
			q = factory.parse("((/(abcd|(def/ghi))))");
			Assert
					.assertEquals(q.toString(),
							"ROOT SELECT(abcd) SELECT(def) SELECT(ghi) JOIN(1) UNION JOIN(1)");
			q = factory.parse("((/(abcd|(def/_/_/((ghi/_*/yyy)|xxx)))))");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(abcd) SELECT(def) SELECT(ghi) SELECT(yyy) JOIN(1..INF) SELECT(xxx) UNION JOIN(3) UNION JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}

	public void testAttribute() {
		XissQueryFactory factory = XissQueryFactory.newInstance();
		IQuery q;
		try {
			q = factory.parse("/elem[@attribute=\"value\"]");
			Assert.assertEquals(q.toString(),
					"ROOT SELECT(elem) AJOIN(attribute=value) JOIN(1)");
			q = factory
					.parse("/elem[@attribute='value']/elem2[@a2=\"v2\"]/elem3");
			Assert
					.assertEquals(
							q.toString(),
							"ROOT SELECT(elem) AJOIN(attribute=value) SELECT(elem2) AJOIN(a2=v2) SELECT(elem3) JOIN(1) JOIN(1) JOIN(1)");
		} catch (QueryParseException e) {
			Assert.fail(e.getMessage());
		}
	}
}